<?php
declare(strict_types=1);

namespace Catequil\ORM\Repository;

use Catequil\Core\Mapping\HasInputFilter;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Repository\RepositoryFactory as RepositoryFactoryInterface;
use Doctrine\Persistence\ObjectRepository;
use Laminas\InputFilter\InputFilterAwareInterface;
use Laminas\InputFilter\InputFilterPluginManager;
use ReflectionClass;
use ReflectionException;

use function spl_object_id;

/**
 * Class RepositoryFactory
 *
 * @package Catequil\ORM\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class RepositoryFactory implements RepositoryFactoryInterface
{
    private InputFilterPluginManager $filters;
    /** @var array<string, ObjectRepository> */
    private array $repositoryList = [];

    /**
     * @param InputFilterPluginManager $filters
     */
    public function __construct(InputFilterPluginManager $filters)
    {
        $this->filters = $filters;
    }

    /**
     * @param EntityManagerInterface $entityManager
     * @param string $entityName
     *
     * @return ObjectRepository
     *
     * @throws ReflectionException
     */
    public function getRepository(EntityManagerInterface $entityManager, $entityName): ObjectRepository
    {
        $repositoryHash = $entityManager->getClassMetadata($entityName)->getName() . spl_object_id($entityManager);

        if (isset($this->repositoryList[$repositoryHash])) {
            return $this->repositoryList[$repositoryHash];
        }

        return $this->repositoryList[$repositoryHash] = $this->createRepository($entityManager, $entityName);
    }

    /**
     * @param EntityManagerInterface $em
     * @param string $entityName
     *
     * @return ObjectRepository
     *
     * @throws ReflectionException
     */
    private function createRepository(EntityManagerInterface $em, string $entityName): ObjectRepository
    {
        $metadata            = $em->getClassMetadata($entityName);
        $repositoryClassName = $metadata->customRepositoryClassName
            ?: $em->getConfiguration()->getDefaultRepositoryClassName();

        $repo = new $repositoryClassName($em, $metadata);

        if ($repo instanceof InputFilterAwareInterface) {
            $reflClass = new ReflectionClass($entityName);
            $reflAttribute = $reflClass->getAttributes(HasInputFilter::class)[0] ?? null;

            if ($reflAttribute) {
                $filterName = $reflAttribute->getArguments()[0];
                $filter = $this->filters->get($filterName);
                $repo->setInputFilter($filter);
            }
        }

        return $repo;
    }
}
